@@ -5,7 +5,8 @@ from inspect import isclass
 from pathlib import Path
 
 from asgi_tools._compat import json_dumps
-from muffin import Application, ResponseFile, ResponseError, ResponseRedirect, Request
+from asgi_tools.utils import to_awaitable
+from muffin import Application, ResponseFile, ResponseRedirect, ResponseError, Request
 from muffin.plugins import BasePlugin
 from muffin_rest.api import API, AUTH
 
@@ -38,6 +39,9 @@ class Plugin(BasePlugin):
 
         'auth_storage': 'localstorage',  # localstorage|cookies
         'auth_storage_name': 'muffin_admin_auth',
+        'app_bar_links': [
+            {'url': '/', 'icon': 'Home', 'title': 'Home'},
+        ],
     }
 
     def __init__(self, *args, **kwargs):
@@ -45,6 +49,7 @@ class Plugin(BasePlugin):
         self.auth: t.Dict = {}
         self.handlers: t.List = []
         self.__login__ = self.__ident__ = page404
+        self.__dashboard__ = None
         super(Plugin, self).__init__(*args, **kwargs)
 
     def setup(self, app: Application, **options):
@@ -59,16 +64,25 @@ class Plugin(BasePlugin):
 
         custom_js = self.cfg.custom_js_url
         custom_css = self.cfg.custom_css_url
+        title = self.cfg.title
+        prefix = self.cfg.prefix
 
         @app.route(self.cfg.prefix)
         async def render_admin(request):
-            if self.cfg.login_url and self.api.authorize:
+            """Render admin page."""
+            if self.api.authorize:
                 auth = await self.api.authorize(request)
                 if not auth:
-                    return ResponseRedirect(self.cfg.login_url)
+                    if self.cfg.login_url:
+                        return ResponseRedirect(self.cfg.login_url)
+
+            data = self.to_ra()
+
+            if self.__dashboard__:
+                data['dashboard'] = await self.__dashboard__(request)
 
             return TEMPLATE.format(
-                admin=self, title=self.app.cfg.name.title(),
+                prefix=prefix, title=title, ra_data=json_dumps(data).decode('utf-8'),
                 custom_js=f"<script src={custom_js} />" if custom_js else '',
                 custom_css=f"<link rel='stylesheet' href={custom_css} />" if custom_css else '',
             )
@@ -116,23 +130,26 @@ class Plugin(BasePlugin):
     def login(self, fn: AUTH) -> AUTH:
         """Register a function to login current user."""
         self.auth['authorizeURL'] = f"{self.cfg.prefix}/login"
-        self.__login__ = fn
+        self.__login__ = to_awaitable(fn)
+        return fn
+
+    def dashboard(self, fn: AUTH) -> AUTH:
+        """Register a function to render dashboard."""
+        self.__dashboard__ = to_awaitable(fn)
         return fn
 
     def get_identity(self, fn: AUTH) -> AUTH:
-        """Register a function to identificate current user."""
+        """Register a function to identificate current user.
+
+        User data: {id, fullName, avatar}
+        """
         self.auth['identityURL'] = f"{ self.cfg.prefix }/ident"
-        self.__ident__ = fn
+        self.__ident__ = to_awaitable(fn)
         return fn
 
     # Serialize to react-admin
     # -------------------------
 
-    @property
-    def json(self) -> str:
-        """Jsonify the plugin."""
-        return json_dumps(self.to_ra()).decode('utf-8')
-
     def to_ra(self) -> t.Dict:
         """Prepare params for react-admin."""
         return {
@@ -142,5 +159,6 @@ class Plugin(BasePlugin):
                 "title": self.cfg.title,
                 "disableTelemetry": True,
             },
+            "appBarLinks": self.cfg.app_bar_links,
             "resources": [res.to_ra() for res in self.handlers],
         }
